#include <J3ML/Rotation.hpp>

namespace J3ML {
    Math::Rotation Math::operator ""_degrees(long double rads) { return {Functions::Radians((float)rads)}; }

    Math::Rotation Math::operator ""_deg(long double rads) { return {Functions::Radians((float)rads)}; }

    Math::Rotation Math::operator ""_radians(long double rads) { return {(float)rads}; }

    Vector2 Math::Rotation::Rotate(const Vector2 &rhs) const {
        float cos_a = Math::Cos(value);
        float sin_a = Math::Sin(value);

        return Vector2(
            rhs.x * cos_a - rhs.y * sin_a,
            rhs.x * sin_a + rhs.y * cos_a);
    }

    Math::Rotation Math::operator ""_rad(long double rads) { return {(float)rads}; }

    Math::Rotation::Rotation() : value(0) {}

    Math::Rotation::Rotation(float value) : value(value) {}

    constexpr Math::Rotation::Rotation(const Vector2 &direction_vector) {
        value = Math::Atan2(direction_vector.y, direction_vector.x);
    }

    constexpr Math::Rotation Math::Rotation::FromDegrees(float degrees) {
        return Rotation(Math::Radians(degrees));
    }

    constexpr Math::Rotation Math::Rotation::FromRadians(float radians) { return Rotation(value);}


    Math::Rotation Math::Rotation::operator+(const Math::Rotation &rhs) const {
        return {value + rhs.value};
    }

    Math::Rotation Math::Rotation::operator-(const Math::Rotation &rhs) const {
        return {value - rhs.value};

    }
}
